/***************************************************************************
 *   Copyright (C) 2006 by Jaap Boender   *
 *   Jaap.Boender@pps.jussieu.fr   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qfile.h>
#include <qdom.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include "kroll_bundle.h"

KrollBundle::KrollBundle ()
{
    m_index = 0;
    m_url = QString ("");
}

int KrollBundle::getFromURL (KURL url)
{
    m_url = url;
    QString tmpFile;
    QDomDocument doc ("weather");
    if (KIO::NetAccess::download (url, tmpFile, 0))
    {
        QFile file (tmpFile);
        if (!file.open (IO_ReadOnly))
        {
            KMessageBox::error (0, "Could not open temporary file " + tmpFile);
            return -1;
        }
        if (!doc.setContent (&file))
        {
            file.close ();
            KMessageBox::error (0, "Downloaded file (" + url.prettyURL () + ") is not a weather information file");
            return -1;
        }
        file.close ();
        KIO::NetAccess::removeTempFile (tmpFile);
    }
    else
    {
        KMessageBox::error (0, KIO::NetAccess::lastErrorString ());
        return -1;
    }
    QDomElement root = doc.documentElement ();
    if (root.tagName () != "weather")
    {
        KMessageBox::error (0, "Downloaded file (" + url.prettyURL () + ") is not a weather information file");
        return -1;
    }
    QDomNode n = root.firstChild ();
    while (!n.isNull ())
    {
        QDomElement e = n.toElement ();
        if (!e.isNull ())
        {
            if (e.tagName () == "index")
                m_index = e.text ().toInt ();
            else if (e.tagName () == "bundle")
            {
                QDomNode n2 = e.firstChild ();
                while (!n2.isNull ())
                {
                    QDomElement e2 = n2.toElement ();
                    if (!e2.isNull ())
                    {
                        if (e2.tagName () == "id")
                            m_bundle_id = e2.text ();
                        else if (e2.tagName () == "description")
                            m_bundle_description = e2.text ();
                        else
                            kdDebug() << "[KrollBundle::getFromURL] unknown tag in <bundle>: " << e2.tagName() << endl;
                    }
                    else
                    {
                        KMessageBox::error (0, "Syntax error in file " + url.prettyURL ());
                        return -1;
                    }
                    n2 = n2.nextSibling ();
                }
            }
            else if (e.tagName () == "packages")
            {
                QDomNode n2 = e.firstChild ();
                while (!n2.isNull ())
                {
                    QDomElement e2 = n2.toElement ();
                    if (!e2.isNull ())
                    {
                        if (e2.tagName () == "total")
                            m_total_packages = e2.text ().toInt ();
                        else if (e2.tagName () == "broken")
                            m_broken_packages = e2.text ().toInt ();
                        else
                            kdDebug() << "[KrollBundle::getFromURL] unknown tag in <packages>: " << e2.tagName() << endl;
                    }
                    else
                    {
                        KMessageBox::error (0, "Syntax error in file " + url.prettyURL ());
                        return -1;
                    }
                    n2 = n2.nextSibling ();
                }
            }
            else if (e.tagName () == "url")
                m_anla_url = e.text ();
            else if (e.tagName () == "architecture")
                m_architecture = e.text ();
            else if (e.tagName () == "date")
                m_date = QDate::fromString (e.text (), Qt::ISODate);
            else if (e.tagName () == "error")
            {
                KMessageBox::error (0, "Error: " + e.text ());
                return -1;
            }
            else
                kdDebug() << "[KrollBundle::getFromURL] unknown tag in <weather>: " << e.tagName () << endl;
        }
        else
        {
            KMessageBox::error (0, "Syntax error in file " + url.prettyURL ());
            return -1;
        }
        n = n.nextSibling ();
    }
    return 0;
}
