/***************************************************************************
 *   Copyright (C) 2006 by Jaap Boender   *
 *   Jaap.Boender@pps.jussieu.fr   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qframe.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qlistbox.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kurlrequesterdlg.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include "kroll_bundle.h"
#include "kroll_configuration.h"
#include "kroll_settings.h"

KrollSettings::KrollSettings(QWidget *parent, const char *name, WFlags f)
    :KDialogBase(IconList, i18n("Preferences"), Default|Ok|Apply|Cancel, Ok, parent, name, f)
{
    QFrame *g = addPage (i18n("General"), i18n("General options"), KGlobal::iconLoader()->loadIcon("kfm",KIcon::Panel,0,false));
    QVBoxLayout *generalLayout = new QVBoxLayout (g, 0, 0);
    m_general = new KrollSettingsGeneral (g);
    generalLayout -> addWidget (m_general);
    QFrame *s = addPage (i18n("Sources"), i18n("Data sources"),
    KGlobal::iconLoader () -> loadIcon ("irkick",KIcon::Panel,0,false));
    QVBoxLayout *sourcesLayout = new QVBoxLayout (s, 0, 0);
    m_sources = new KrollSettingsSources (s);
    connect (m_sources -> pbAdd, SIGNAL(clicked()), this, SLOT(addURL()));
    connect (m_sources -> pbRemove, SIGNAL(clicked()), this, SLOT(removeURL()));
    sourcesLayout -> addWidget (m_sources);
}

void KrollSettings::updateConfiguration()
{
    if (m_general->rbOneSource->isChecked ())
        Config().m_index_method = OneSource;
    else
        Config().m_index_method = Average;
    Config().m_sources.clear ();
    for (uint i = 0; i < m_sources -> lbSources -> count(); i++)
        Config().m_sources.append (m_sources -> lbSources -> text (i));
}

void KrollSettings::updateDialog()
{
    if (Config().m_index_method == OneSource)
    {
        m_general -> rbOneSource -> setChecked (true);
        m_general -> rbAverage -> setChecked (false);
    }
    else
    {
        m_general -> rbOneSource -> setChecked (false);
        m_general -> rbAverage -> setChecked (true);
    }
    m_sources -> lbSources -> clear ();
    for (uint i = 0; i < Config().m_sources.count (); i++)
        m_sources -> lbSources -> insertItem (Config().m_sources.at (i));
}

#include "kroll_settings.moc"


/*!
    \fn KrollSettings::addURL ()
 */
void KrollSettings::addURL ()
{
    KURLRequesterDlg dlg ("", this, 0);
    KrollBundle *b = new KrollBundle ();
    KURL u = dlg.getURL (QString::null, 0, "Source URL");
    if (b -> getFromURL (u) >= 0)
        m_sources -> lbSources -> insertItem (b -> URL().url());
}

void KrollSettings::removeURL ()
{
    m_sources -> lbSources -> removeItem (m_sources -> lbSources -> currentItem ());
}
