/***************************************************************************
 *   Copyright (C) 2006 by Jaap Boender   *
 *   Jaap.Boender@pps.jussieu.fr   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlayout.h>
#include <qptrlist.h>
#include <kstandarddirs.h>
#include <kicontheme.h>
#include <kiconeffect.h>
#include <krun.h>
#include <kdebug.h>

#include "kroll_iconviewitem.h"
#include "kroll_widget.h"

KrollWidget::KrollWidget(QWidget *parent, const char *name)
 : QWidget(parent, name)
{
    m_image_index = 1;
    m_selected_item = QString ("");
    m_old_width = m_old_height = m_old_index = -1;
    m_imagePath = QString ("kroll/pics/weather-%1.png");
    m_popup = new KPopupFrame (this);
    QVBoxLayout *vbox = new QVBoxLayout (m_popup, 0, 0);
    m_iconView = new KIconView (m_popup);
    m_iconView -> setSelectionMode (QIconView::NoSelection);
    m_iconView -> setItemTextPos (QIconView::Right);
    m_iconView -> setHScrollBarMode (QScrollView::AlwaysOff);
    m_iconView -> setItemsMovable (false);
    m_iconView -> setMaxItemWidth (200);
    m_iconView -> setWordWrapIconText (true);
    connect (m_iconView, SIGNAL(clicked(QIconViewItem *, const QPoint &)), this, SLOT(iconClicked(QIconViewItem *, const QPoint &)));
    vbox -> addWidget (m_iconView, 1);
}

void KrollWidget::clearBundles ()
{
    QPtrListIterator<KrollBundle> it (m_bundles);
    while (KrollBundle *b = it.current ())
    {
        ++it;
        removeBundle (b);
    }
}

void KrollWidget::paintEvent (QPaintEvent *e)
{
    bitBlt (this, 0, 0, &m_pixmap, 0, 0);
}

void KrollWidget::resizeEvent (QResizeEvent *e)
{
    draw ();
    repaint ();
}

void KrollWidget::draw ()
{
    if ((width () == m_old_width) && (height () == m_old_height) && (m_image_index == m_old_index))
        return;
    m_old_width = width ();
    m_old_height = height ();
    m_old_index = m_image_index;
    QImage i = loadImage (m_image_index);
    if (i.isNull ())
        return;
    i = i.convertDepth (32, 0);
    if (i.isNull ())
        return;
    int mw = QMIN (width(), height ());
    if (!m_pixmap.convertFromImage (i.smoothScale (mw, mw).convertDepth (32), 0))
        return;
}

QImage KrollWidget::loadImage (int index)
{
    if ((index < 1) || (index > 5))
        index = 1;
    QString imageFile = locate ("data", m_imagePath.arg (index));
    if (imageFile.isNull ())
        kdFatal() << "Could not locate image file " << imageFile << "; exiting." << endl;
    QImage image (imageFile);
    KIconEffect iconeffect;
    image = iconeffect.apply (image, KIcon::Panel, KIcon::DefaultState);
    return image;
}


void KrollWidget::calculateIndex ()
{
    int total = 0;
    m_iconView -> clear ();
    if (m_bundles.isEmpty ())
        return;
    QPtrListIterator <KrollBundle> it (m_bundles);
    KrollBundle *b;
    while (b = it.current ())
    {
        ++it;
        KrollIconViewItem *i = new KrollIconViewItem (m_iconView, b);
        total += b -> index ();
    }
    if (m_index_method == OneSource)
    {
        QIconViewItem *item = m_iconView -> findItem (m_selected_item + " ");
        if (!item)
            return;
        if (item -> rtti () != KROLL_ICONVIEWITEM_RTTI)
            return;
        ((KrollIconViewItem *) item) -> setHighlighted (true);
        m_image_index = ((KrollIconViewItem *) item) -> bundle() -> index ();
    }
    else
        m_image_index = total / m_bundles.count ();
    draw ();
    repaint ();
}

/*!
    \fn KrollWidget::showPopup ()
 */
void KrollWidget::showPopup (QPoint pos)
{
    // force the popup window to be as large as its children
    int height = m_iconView->spacing() + 10; // don't know why the 10; it's not the margin of the KPopupFrame
    QPoint global = mapToGlobal (pos);
    for (QIconViewItem *i = m_iconView->firstItem(); i; i = i->nextItem())
        height += i->height() + m_iconView->spacing();
    if (height < 300)
    {
        m_popup -> setMinimumHeight (height);
        m_popup -> setMaximumHeight (height);
        m_popup -> exec (global.x (), global.y () - height);
    }
    else
        m_popup -> exec (global.x (), global.y () - m_iconView->height());
}

/*!
    \fn KrollWidget::iconClicked (QIconViewItem *, const QPoint &)
 */
void KrollWidget::iconClicked (QIconViewItem *i, const QPoint &pos)
{
    // pos: global
    // textRect(), pixmapRect(): relative to either i (true, default) or m_iconView (false)
    if (!i)
        return;
    if (i -> rtti() != KROLL_ICONVIEWITEM_RTTI)
        return;
    KrollIconViewItem *b = (KrollIconViewItem *) i;
    QRect textRect (m_iconView->mapToGlobal (b->textRect (false).topLeft ()), m_iconView->mapToGlobal (b->textRect (false).bottomRight ())), pixmapRect (m_iconView->mapToGlobal (b->pixmapRect (false).topLeft ()), m_iconView->mapToGlobal (b->pixmapRect (false).bottomRight ()));
    if ((m_index_method == OneSource) && (pixmapRect.contains (pos)))
    {
       setSelectedItem (b->bundle()->description());
       calculateIndex ();
    }
    else
        (void) new KRun (b->bundle()->anlaURL());
    m_popup -> hide ();
}

void KrollWidget::setSelectedItem (QString s)
{
    QIconViewItem *old = m_iconView -> findItem (m_selected_item + " ");
    if ((old) && (old -> rtti () == KROLL_ICONVIEWITEM_RTTI))
        ((KrollIconViewItem *) old) -> setHighlighted (false);
    QIconViewItem *item = m_iconView -> findItem (s + " ");
    if (!item)
        item = m_iconView -> firstItem ();
    if (!item)
    {
        m_selected_item = "";
        return;
    }
    if (item -> rtti () == KROLL_ICONVIEWITEM_RTTI)
    {
        ((KrollIconViewItem *) item) -> setHighlighted (true);
        m_selected_item = ((KrollIconViewItem *) item) -> bundle () -> description ();
    }
    else
        m_selected_item = item -> text ();
}
